package org.vaadin.example.quartz;

import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.spi.JobFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.enterprise.event.Observes;
import javax.inject.Inject;
import org.vaadin.example.quartz.event.StartSchedulerEvent;

/**
 *
 * @author wassimz
 */
@Startup
@Singleton
public class SchedulerBean {

    private static final Logger log = LoggerFactory.getLogger(SchedulerBean.class);

    private Scheduler scheduler;

    @Inject
    private JobFactory cdiJobFactory;

    @PostConstruct
    public void scheduleJobs() {
        try {
            scheduler = new StdSchedulerFactory().getScheduler();
            scheduler.setJobFactory(cdiJobFactory);
        } catch (SchedulerException e) {
            log.error("Error while creating scheduler", e);
        }
    }

    /**
     * Start the scheduler once we receive the event
     *
     * @param event
     */
    public void startSchedulerAndExecuteJob(@Observes StartSchedulerEvent event) {
        try {
            
            JobKey jobKey = JobKey.jobKey("job", "jobs");
            JobDetail job = JobBuilder
                    .newJob(QuartzJob.class)
                    .withIdentity(jobKey)
                    .build();

            TriggerKey tk = TriggerKey.triggerKey("trigger", "jobs");
            Trigger trigger = TriggerBuilder
                    .newTrigger()
                    .withIdentity(tk)
                    .startNow()
                    .withSchedule(SimpleScheduleBuilder
                            .repeatMinutelyForever(event.getRepeatTimeInMinutes()))
                    .build();

            scheduler.scheduleJob(job, trigger);
            scheduler.start();
            
        } catch (SchedulerException ex) {
            log.error("error while starting the scheduler", ex);
        }

    }

    @PreDestroy
    public void stopJobs() {
         if (scheduler != null) {
            try {
                scheduler.shutdown(false);
            } catch (SchedulerException e) {
                log.error("Error while closing scheduler", e);
            }
        }
    }
}
